/* Process model C form file: fault_pr.pr.c */
/* Portions of this file copyright 1992, 1996 by MIL 3, Inc. */



/* OPNET system definitions */
#include <opnet.h>
#include "/homes/psebos/op_models/fault_pr.pr.h"
FSM_EXT_DECS



/* Header block */
#define FAIL (op_intrpt_type () == OPC_INTRPT_FAIL)
#define RECOVER (op_intr_type () == OPC_INTRPT_RECOVER)
#define FAIL_CODE 1
#define RECOVER_CODE 0
typedef struct {
  float time;
  int user_id;
  int type;
} Script_Inst;
#if !defined (VOSD_NO_FIN)
#undef	BIN
#undef	BOUT
#define	BIN		_fstack_local_info.last_line_passed = __LINE__ - _block_origin;
#define	BOUT	BIN
#define	BINIT	_fstack_local_info.last_line_passed = 0; _block_origin = __LINE__;
#else
#define	BINIT
#endif /* #if !defined (VOSD_NO_FIN) */



/* State variable definitions */
typedef struct
	{
	FSM_SYS_STATE
	} fault_pr_state;

#define pr_state_ptr            		((fault_pr_state*) SimI_Mod_State_Ptr)

/* This macro definition will define a local variable called	*/
/* "op_sv_ptr" in each function containing a FIN statement.	*/
/* This variable points to the state variable data structure,	*/
/* and can be used from a C debugger to display their values.	*/
#undef FIN_PREAMBLE
#define FIN_PREAMBLE	fault_pr_state *op_sv_ptr = pr_state_ptr;




/* Process model interrupt handling procedure */



void
fault_pr ()
	{
	List*	event_list_ptr;
	List*	line_list_ptr;
	int		i,list_size;
	Vartype*	string,field0,field2;
	List*	field_list_ptr;
	Script_Inst*	inst_ptr;
	Vartype*	script_name;
	int object_id;
	int _block_origin = 0;


	FSM_ENTER (fault_pr)

	FSM_BLOCK_SWITCH
		{
		/*---------------------------------------------------------*/
		/** state (init) enter executives **/
		FSM_STATE_ENTER_FORCED (0, state0_enter_exec, "init", "fault_pr () [init enter execs]")
			{
			/* Obtain name of script file from the process' attribute. */ 
			op_ima_obj_attr_get (op_id_self (), "script name", script_name); 
			
			/* Create and init list to hold failure/recovery instructions. */
			event_list_ptr = op_prg_list_create ();   
			
			/* Load the file contents into a list. */
			line_list_ptr = op_prg_gdf_read (script_name);
			
			/* Loop through the lines of the list and parse each line. */ 
			list_size = op_prg_list_size (line_list_ptr); 
			for (i = 0; i < list_size; i++) 
			  { 
			  /* Obtain the i_th line in the file. */ 
			  string = op_prg_list_access (line_list_ptr, i); 
			
			  /* Decompose the line in to fields. */ 
			  field_list_ptr = op_prg_str_decomp (string, " ,/\t"); 
			
			  /* If the line does not have three fields or starts with '#' */ 
			  /* then skip it and try the next. */ 
			  if (op_prg_list_size (field_list_ptr) != 3) 
			      goto dealloc; 
			  field0 = op_prg_list_access (field_list_ptr, 0); 
			  if (field0 [0] == '#') goto dealloc; 
			
			
			  /* Allocate a structure to hold the event instruction. */ 
			  inst_ptr = (Script_Inst*) op_prg_mem_alloc (sizeof (Script_Inst)); 
			
			  /* Fill in the instruction. */ 
			  inst_ptr->time = atof (field0); 
			  inst_ptr->user_id = atoi (op_prg_list_access (field_list_ptr, 1)); 
			  field2 = op_prg_list_access (field_list_ptr, 2); 
			  if (strcmp ("failure", field2) == 0) 
			    inst_ptr->type = FAIL_CODE; 
			  else inst_ptr->type = RECOVER_CODE; 
			
			  /* Place the instruction on the end of the list. */ 
			  op_prg_list_insert (event_list_ptr, inst_ptr, OPC_LISTPOS_TAIL); 
			
			  /* Deallocate the elements in the list and then the list itself. */ 
			  dealloc: 
			  op_prg_list_free (field_list_ptr); 
			  op_prg_mem_free (field_list_ptr); 
			  } 
			 
			/* Deallocate the elements in the list and then the list itself. */ 
			op_prg_list_free (line_list_ptr); 
			op_prg_mem_free (line_list_ptr); 
			}


		/** state (init) exit executives **/
		FSM_STATE_EXIT_FORCED (0, state0_exit_exec, "init", "fault_pr () [init exit execs]")
			{
			}


		/** state (init) transition processing **/
		FSM_TRANSIT_FORCE (3, state3_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (idle) enter executives **/
		FSM_STATE_ENTER_UNFORCED (1, state1_enter_exec, "idle", "fault_pr () [idle enter execs]")
			{
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (3,fault_pr)


		/** state (idle) exit executives **/
		FSM_STATE_EXIT_UNFORCED (1, state1_exit_exec, "idle", "fault_pr () [idle exit execs]")
			{
			}


		/** state (idle) transition processing **/
		FSM_INIT_COND (FAIL)
		FSM_TEST_COND (RECOVER)
		FSM_DFLT_COND
		FSM_TEST_LOGIC ("idle")

		FSM_TRANSIT_SWITCH
			{
			FSM_CASE_TRANSIT (0, 2, state2_enter_exec, ;)
			FSM_CASE_TRANSIT (1, 2, state2_enter_exec, ;)
			FSM_CASE_TRANSIT (2, 1, state1_enter_exec, ;)
			}
		/*---------------------------------------------------------*/



		/** state (intr) enter executives **/
		FSM_STATE_ENTER_FORCED (2, state2_enter_exec, "intr", "fault_pr () [intr enter execs]")
			{
			/* This process just received a self intrpt that signals a failure */ 
			/* or recovery action. Obtain the top instruction from the list. */ 
			inst_ptr = (Script_Inst*) op_prg_list_remove (event_list_ptr, OPC_LISTPOS_HEAD); 
			
			/* Convert user ID to object ID. */ 
			object_id = user_id_to_object_id_convert 
			                             (inst_ptr->user_id); 
			
			/* Set the object's "condition" attribute to the specified value. */ 
			if (inst_ptr->type == FAIL_CODE) 
			   op_ima_obj_attr_set (object_id, "condition", 
			                                 OPC_BOOLINT_DISABLED); 
			else 
			   op_ima_obj_attr_set (object_id, "condition",
			                                  OPC_BOOLINT_ENABLED); 
			
			/* Deallocate instruction. */ 
			op_prg_mem_free (inst_ptr); 
			
			/* Transition to next state, where next interrupt is scheduled. */ 
			}


		/** state (intr) exit executives **/
		FSM_STATE_EXIT_FORCED (2, state2_exit_exec, "intr", "fault_pr () [intr exit execs]")
			{
			}


		/** state (intr) transition processing **/
		FSM_TRANSIT_FORCE (3, state3_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (sche) enter executives **/
		FSM_STATE_ENTER_FORCED (3, state3_enter_exec, "sche", "fault_pr () [sche enter execs]")
			{
			/* Obtain the next instruction from the list. */ 
			inst_ptr = (Script_Inst*) op_prg_list_access (event_list_ptr, OPC_LISTPOS_HEAD); 
			
			/* Schedule a self interrupt for the time. */ 
			op_intrpt_schedule_self (inst_ptr->time, 0); 
			}


		/** state (sche) exit executives **/
		FSM_STATE_EXIT_FORCED (3, state3_exit_exec, "sche", "fault_pr () [sche exit execs]")
			{
			}


		/** state (sche) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		}


	FSM_EXIT (0,fault_pr)
	}




void
fault_pr_svar (prs_ptr,var_name,var_p_ptr)
	fault_pr_state		*prs_ptr;
	char			*var_name, **var_p_ptr;
	{

	FIN (fault_pr_svar (prs_ptr))

	*var_p_ptr = VOS_NIL;

	FOUT;
	}




void
fault_pr_diag ()
	{
	List*	event_list_ptr;
	List*	line_list_ptr;
	int		i,list_size;
	Vartype*	string,field0,field2;
	List*	field_list_ptr;
	Script_Inst*	inst_ptr;
	Vartype*	script_name;
	int object_id;
	int _block_origin;

	FIN (fault_pr_diag ())

	BINIT

	FOUT;
	}




void
fault_pr_terminate ()
	{
	List*	event_list_ptr;
	List*	line_list_ptr;
	int		i,list_size;
	Vartype*	string,field0,field2;
	List*	field_list_ptr;
	Script_Inst*	inst_ptr;
	Vartype*	script_name;
	int object_id;
	int _block_origin;

	FIN (fault_pr_terminate ())

	BINIT

	FOUT;
	}



Compcode
fault_pr_init (pr_state_pptr)
	fault_pr_state			**pr_state_pptr;
	{
	int _block_origin = 0;
	static VosT_Cm_Obtype	obtype = OPC_NIL;

	extern int				Vos_Nop ();

	FIN (fault_pr_init (pr_state_pptr))

	if (obtype == OPC_NIL)
		{
		if (Vos_Catmem_Register ("proc state vars (fault_pr)",
			sizeof (fault_pr_state), Vos_Nop, &obtype) == VOSC_FAILURE)
			FRET (OPC_COMPCODE_FAILURE)
		}

	if ((*pr_state_pptr = (fault_pr_state*) Vos_Catmem_Alloc (obtype, 1)) == OPC_NIL)
		FRET (OPC_COMPCODE_FAILURE)
	else
		{
		(*pr_state_pptr)->current_block = 0;
		FRET (OPC_COMPCODE_SUCCESS)
		}
	}



enum { _block_origin = __LINE__ };
